# Generated by Django 5.1.4 on 2025-02-05 16:43

from django.db import migrations, models

import asymmetric_jwt_auth.models


class Migration(migrations.Migration):

    dependencies = [
        ("asymmetric_jwt_auth", "0005_auto_20210304_1116"),
    ]

    operations = [
        migrations.AddField(
            model_name="jwksendpointtrust",
            name="last_used_on",
            field=models.DateTimeField(
                blank=True, null=True, verbose_name="Last Used On"
            ),
        ),
        migrations.AlterField(
            model_name="publickey",
            name="key",
            field=models.TextField(
                help_text="The user's RSA/Ed25519 public key",
                validators=[asymmetric_jwt_auth.models.validate_public_key],
                verbose_name="Public Key",
            ),
        ),
    ]
